@@ -136,7 +136,8 @@ def create_transport(url, connection_attempts=None):
 
     elif scheme == 'azure':
         valid_str_keys = ['access_key', 'tenant_id', 'client_id', 'client_secret']
-        extras = _get_extras(parse_qs(parsed.query), valid_str_keys=valid_str_keys)
+        valid_bool_keys = ['allow_directory_access']
+        extras = _get_extras(parse_qs(parsed.query), valid_str_keys=valid_str_keys, valid_bool_keys=valid_bool_keys)
 
         t = TransportAzure(base=base, host=host, connection_attempts=connection_attempts, **extras)
 
@@ -149,11 +150,15 @@ def create_transport(url, connection_attempts=None):
 class FileStore(object):
     def __init__(self, *transport_urls, connection_attempts=None):
         self.log = logging.getLogger('assemblyline.transport')
+        self.transport_urls = transport_urls
         self.transports = [create_transport(url, connection_attempts) for url in transport_urls]
         self.local_transports = [
             t for t in self.transports if isinstance(t, TransportLocal)
         ]
 
+    def __eq__(self, obj: FileStore) -> bool:
+        return self.transport_urls == obj.transport_urls
+
     def __enter__(self):
         return self
 
