@@ -2,6 +2,8 @@ from typing import Dict, List
 
 from assemblyline import odm
 from assemblyline.odm.models.service import EnvironmentVariable
+from assemblyline.odm.models.service_delta import DockerConfigDelta
+
 
 AUTO_PROPERTY_TYPE = ['access', 'classification', 'type', 'role', 'remove_role']
 
@@ -366,7 +368,6 @@ class Expiry(odm.Model):
     delete_workers = odm.Integer(description="Worker processes for file storage deletes.")
     iteration_max_tasks = odm.Integer(description="How many query chunks get run per iteration.")
     delete_batch_size = odm.Integer(description="How large a batch get deleted per iteration.")
-    archive_batch_size = odm.Integer(description="How large a batch get archived per iteration.")
 
 
 DEFAULT_EXPIRY = {
@@ -378,7 +379,6 @@ DEFAULT_EXPIRY = {
     'delete_workers': 2,
     'iteration_max_tasks': 20,
     'delete_batch_size': 2000,
-    'archive_batch_size': 5000,
 }
 
 
@@ -501,6 +501,19 @@ DEFAULT_METRICS = {
 }
 
 
+@odm.model(index=False, store=False, description="Malware Archive Configuration")
+class Archiver(odm.Model):
+    minimum_required_services: List[str] = odm.List(
+        odm.keyword(),
+        default=[],
+        description="List of minimum required service before archiving takes place")
+
+
+DEFAULT_ARCHIVER = {
+    'minimum_required_services': []
+}
+
+
 @odm.model(index=False, store=False, description="Redis Configuration")
 class Redis(odm.Model):
     nonpersistent: RedisServer = odm.Compound(RedisServer, default=DEFAULT_REDIS_NP,
@@ -523,9 +536,15 @@ class Mount(odm.Model):
     privileged_only: bool = odm.Boolean(default=False,
                                         description="Should this mount only be available for privileged services?")
 
-    # ConfigMap-specific
-    config_map: str = odm.Optional(odm.Keyword(description="Name of ConfigMap (Kubernetes only)"))
-    key: str = odm.Optional(odm.Keyword(), description="Key of ConfigMap (Kubernetes only)")
+    # Kubernetes-specific
+    resource_type: str = odm.Enum(default='volume', values=['secret', 'configmap', 'volume'],
+                                  description="Type of mountable Kubernetes resource")
+    resource_name: str = odm.Optional(odm.Keyword(), description="Name of resource (Kubernetes only)")
+    resource_key: str = odm.Optional(odm.Keyword(), description="Key of ConfigMap/Secret (Kubernetes only)")
+
+    # TODO: Deprecate in next major change in favour of general configuration above for mounting Kubernetes resources
+    config_map: str = odm.Optional(odm.Keyword(), description="Name of ConfigMap (Kubernetes only, deprecated)")
+    key: str = odm.Optional(odm.Keyword(), description="Key of ConfigMap (Kubernetes only, deprecated)")
 
 
 @odm.model(index=False, store=False,
@@ -571,6 +590,34 @@ DEFAULT_SCALER = {
 }
 
 
+@odm.model(index=False, store=False)
+class RegistryConfiguration(odm.Model):
+    name: str = odm.Text(description="Name of container registry")
+    proxies: Dict = odm.Optional(odm.Mapping(odm.Text()),
+                                 description="Proxy configuration that is passed to Python Requests")
+
+
+@odm.model(index=False, store=False)
+class Updater(odm.Model):
+    job_dockerconfig: DockerConfigDelta = odm.Compound(
+        DockerConfigDelta, description="Container configuration used for service registration/updates")
+    registry_configs: List = odm.List(odm.Compound(RegistryConfiguration),
+                                      description="Configurations to be used with container registries")
+
+
+DEFAULT_UPDATER = {
+    'job_dockerconfig': {
+        'cpu_cores': 1,
+        'ram_mb': 1024,
+        'ram_mb_min': 128,
+    },
+    'registry_configs': [{
+        'name': 'registry.hub.docker.com',
+        'proxies': {}
+    }]
+}
+
+
 @odm.model(index=False, store=False)
 class VacuumSafelistItem(odm.Model):
     name = odm.Keyword()
@@ -592,6 +639,7 @@ class Vacuum(odm.Model):
     worker_threads: int = odm.Integer()
     worker_rollover: int = odm.Integer()
     minimum_classification: str = odm.Keyword()
+    ingest_type = odm.keyword()
 
 
 DEFAULT_VACUUM = dict(
@@ -608,12 +656,15 @@ DEFAULT_VACUUM = dict(
     worker_threads=50,
     worker_rollover=1000,
     minimum_classification='U',
+    ingest_type='VACUUM',
 )
 
 
 @odm.model(index=False, store=False, description="Core Component Configuration")
 class Core(odm.Model):
     alerter: Alerter = odm.Compound(Alerter, default=DEFAULT_ALERTER, description="Configuration for Alerter")
+    archiver: Archiver = odm.Compound(Archiver, default=DEFAULT_ARCHIVER,
+                                      description="Configuration for the permanent submission archive")
     dispatcher: Dispatcher = odm.Compound(Dispatcher, default=DEFAULT_DISPATCHER,
                                           description="Configuration for Dispatcher")
     expiry: Expiry = odm.Compound(Expiry, default=DEFAULT_EXPIRY, description="Configuration for Expiry")
@@ -622,80 +673,48 @@ class Core(odm.Model):
                                     description="Configuration for Metrics Collection")
     redis: Redis = odm.Compound(Redis, default=DEFAULT_REDIS, description="Configuration for Redis instances")
     scaler: Scaler = odm.Compound(Scaler, default=DEFAULT_SCALER, description="Configuration for Scaler")
+    updater: Updater = odm.Compound(Updater, default=DEFAULT_UPDATER, description="Configuration for Updater")
     vacuum: Vacuum = odm.Compound(Vacuum, default=DEFAULT_VACUUM, description="Configuration for Vacuum")
 
 
 DEFAULT_CORE = {
     "alerter": DEFAULT_ALERTER,
+    "archiver": DEFAULT_ARCHIVER,
     "dispatcher": DEFAULT_DISPATCHER,
     "expiry": DEFAULT_EXPIRY,
     "ingester": DEFAULT_INGESTER,
     "metrics": DEFAULT_METRICS,
     "redis": DEFAULT_REDIS,
     "scaler": DEFAULT_SCALER,
+    "updater": DEFAULT_UPDATER,
 }
 
 
-@odm.model(index=False, store=False, description="Parameters associated to ILM Policies")
-class ILMParams(odm.Model):
-    warm = odm.Integer(description="How long, per unit of time, should a document remain in the 'warm' tier?")
-    cold = odm.Integer(description="How long, per unit of time, should a document remain in the 'cold' tier?")
-    delete = odm.Integer(description="How long, per unit of time, should a document remain before being deleted?")
-    unit = odm.Enum(['d', 'h', 'm'], description="Unit of time used by `warm`, `cold`, `delete` phases")
+@odm.model(index=False, store=False, description="Datastore Archive feature configuration")
+class Archive(odm.Model):
+    enabled = odm.Boolean(description="Are we enabling Achiving features across indices?")
+    indices = odm.List(odm.Keyword(), description="List of indices the ILM Applies to")
 
 
-DEFAULT_ILM_PARAMS = {
-    "warm": 5,
-    "cold": 15,
-    "delete": 30,
-    "unit":  "d"
-}
-
-
-@odm.model(index=False, store=False, description="ILM Policies for Core Indices")
-class ILMIndexes(odm.Model):
-    alert = odm.Compound(ILMParams, default=DEFAULT_ILM_PARAMS, description="ILM for 'alert' index")
-    error = odm.Compound(ILMParams, default=DEFAULT_ILM_PARAMS, description="ILM for 'error' index")
-    file = odm.Compound(ILMParams, default=DEFAULT_ILM_PARAMS, description="ILM for 'file' index")
-    result = odm.Compound(ILMParams, default=DEFAULT_ILM_PARAMS, description="ILM for 'result' index")
-    submission = odm.Compound(ILMParams, default=DEFAULT_ILM_PARAMS, description="ILM for 'submission' index")
-
-
-DEFAULT_ILM_INDEXES = {
-    'alert': DEFAULT_ILM_PARAMS,
-    'error': DEFAULT_ILM_PARAMS,
-    'file': DEFAULT_ILM_PARAMS,
-    'result': DEFAULT_ILM_PARAMS,
-    'submission': DEFAULT_ILM_PARAMS,
-}
-
-
-@odm.model(index=False, store=False, description="Index Lifecycle Management")
-class ILM(odm.Model):
-    enabled = odm.Boolean(description="Are we enabling ILM across indices?")
-    days_until_archive = odm.Integer(description="Days until documents get archived")
-    indexes = odm.Compound(ILMIndexes, default=DEFAULT_ILM_INDEXES, description="Index-specific ILM policies")
-    update_archive = odm.Boolean(description="Do we want to update documents in the archive?")
-
-
-DEFAULT_ILM = {
-    "days_until_archive": 15,
+DEFAULT_ARCHIVE = {
     "enabled": False,
-    "indexes": DEFAULT_ILM_INDEXES,
-    "update_archive": False
+    "indices": ['file', 'submission', 'result'],
 }
 
 
 @odm.model(index=False, store=False, description="Datastore Configuration")
 class Datastore(odm.Model):
     hosts: List[str] = odm.List(odm.Keyword(), description="List of hosts used for the datastore")
-    ilm = odm.Compound(ILM, default=DEFAULT_ILM, description="Index Lifecycle Management Policy")
+    archive = odm.Compound(Archive, default=DEFAULT_ARCHIVE, description="Datastore Archive feature configuration")
+    cache_dtl = odm.Integer(
+        default=5, description="Default cache lenght for computed indices (submission_tree, submission_summary...")
     type = odm.Enum({"elasticsearch"}, description="Type of application used for the datastore")
 
 
 DEFAULT_DATASTORE = {
     "hosts": ["http://elastic:devpass@localhost:9200"],
-    "ilm": DEFAULT_ILM,
+    "archive": DEFAULT_ARCHIVE,
+    "cache_dtl": 5,
     "type": "elasticsearch",
 }
 
@@ -720,11 +739,13 @@ DEFAULT_DATASOURCES = {
 
 @odm.model(index=False, store=False, description="Filestore Configuration")
 class Filestore(odm.Model):
+    archive: List[str] = odm.List(odm.Keyword(), description="List of filestores used for malware archive")
     cache: List[str] = odm.List(odm.Keyword(), description="List of filestores used for caching")
     storage: List[str] = odm.List(odm.Keyword(), description="List of filestores used for storage")
 
 
 DEFAULT_FILESTORE = {
+    "archive": ["s3://al_storage_key:Ch@ngeTh!sPa33w0rd@localhost:9000?s3_bucket=al-archive&use_ssl=False"],
     "cache": ["s3://al_storage_key:Ch@ngeTh!sPa33w0rd@localhost:9000?s3_bucket=al-cache&use_ssl=False"],
     "storage": ["s3://al_storage_key:Ch@ngeTh!sPa33w0rd@localhost:9000?s3_bucket=al-storage&use_ssl=False"]
 }
@@ -807,7 +828,6 @@ class ServiceRegistry(odm.Model):
 class Services(odm.Model):
     categories: List[str] = odm.List(odm.Keyword(), description="List of categories a service can be assigned to")
     default_timeout: int = odm.Integer(description="Default service timeout time in seconds")
-    min_service_workers: int = odm.Integer(description="The minimum number of service instances to always be running.")
     stages: List[str] = odm.List(odm.Keyword(), description="List of execution stages a service can be assigned to")
     image_variables: Dict[str, str] = odm.Mapping(odm.Keyword(default=''),
                                                   description="Substitution variables for image paths "
@@ -840,7 +860,6 @@ class Services(odm.Model):
 DEFAULT_SERVICES = {
     "categories": SERVICE_CATEGORIES,
     "default_timeout": 60,
-    "min_service_workers": 0,
     "stages": SERVICE_STAGES,
     "image_variables": {},
     "update_image_variables": {},
@@ -866,13 +885,14 @@ class System(odm.Model):
 DEFAULT_SYSTEM = {
     "constants": "assemblyline.common.constants",
     "organisation": "ACME",
-    "type": 'production'
+    "type": 'production',
 }
 
 
 @odm.model(index=False, store=False, description="Statistics")
 class Statistics(odm.Model):
-    alert: List[str] = odm.List(odm.Keyword(), description="Fields used to generate statistics in the Alerts page")
+    alert: List[str] = odm.List(odm.Keyword(),
+                                description="Fields used to generate statistics in the Alerts page")
     submission: List[str] = odm.List(odm.Keyword(),
                                      description="Fields used to generate statistics in the Submissions page")
 
@@ -959,6 +979,7 @@ class UI(odm.Model):
     read_only_offset: str = odm.Keyword(
         default="", description="Offset of the read only mode for all paging and searches")
     rss_feeds: List[str] = odm.List(odm.Keyword(), default=[], description="List of RSS feeds to display on the UI")
+    services_feed: str = odm.Keyword(description="Feed of all the services available on AL")
     secret_key: str = odm.Keyword(description="Flask secret key to store cookies, etc.")
     session_duration: int = odm.Integer(description="Duration of the user session before the user has to login again")
     statistics: Statistics = odm.Compound(Statistics, default=DEFAULT_STATISTICS,
@@ -1001,13 +1022,16 @@ DEFAULT_UI = {
         "https://alpytest.blob.core.windows.net/pytest/stable.json",
         "https://alpytest.blob.core.windows.net/pytest/services.json"
     ],
+    "services_feed": "https://alpytest.blob.core.windows.net/pytest/services.json",
     "secret_key": "This is the default flask secret key... you should change this!",
     "session_duration": 3600,
     "statistics": DEFAULT_STATISTICS,
     "tos": None,
     "tos_lockout": False,
     "tos_lockout_notify": None,
-    "url_submission_headers": {},
+    "url_submission_headers": {
+        "User-Agent": "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36"
+    },
     "url_submission_proxies": {},
     "validate_session_ip": True,
     "validate_session_useragent": True,
