@@ -5,7 +5,7 @@ from assemblyline.odm.models.ontology.results import Antivirus, Process, Sandbox
 from assemblyline.odm.models.ontology.filetypes import PE
 
 Classification = forge.get_classification()
-ODM_VERSION = "1.2"
+ODM_VERSION = "1.3"
 
 
 @odm.model(description="File Characteristics")
@@ -53,6 +53,7 @@ class Results(odm.Model):
     tags = odm.Optional(odm.Mapping(odm.List(odm.Any())),
                         description="Tags raised during analysis. Refer to [Tagging](../../tagging/)")
     heuristics = odm.Optional(odm.List(odm.Compound(Heuristics)), description="Heuristics raised during analysis")
+    score = odm.Optional(odm.Integer(description="The score assigned to the file"))
 
 
 @odm.model(index=False, store=False, description="Service Details")
@@ -73,6 +74,7 @@ class Submission(odm.Model):
                                               description="Submitted classification")
     submitter = odm.Optional(odm.Keyword(), description="Submitter")
     retention_id = odm.Optional(odm.Keyword(), description="Reference to knowledge base for long-term data retention.")
+    max_score = odm.Optional(odm.Integer(description="The highest file score of the submission"))
 
 
 # ResultOntology
